@@ -5,14 +5,13 @@
 describe('35 Lemonldap::NG Manager', function() {
 
   it('should be able to restore an old configuration', function() {
-    browser.get('/#/confs/1');
+    browser.get('/#!/confs/1');
     element(by.id('save')).click();
     element(by.id('longtextinput')).sendKeys('Restore conf 1');
     element(by.id('saveok')).click();
-    browser.sleep(500);
     element(by.id('messageok')).click();
-    browser.sleep(500);
     element(by.id('forcesave')).click();
+    element(by.id('save')).click();
     element(by.id('longtextinput')).sendKeys('Force to restore conf 1');
     element(by.id('saveok')).click();
     element(by.id('messageok')).click();
