@@ -4,7 +4,7 @@ use strict;
 use Lemonldap::NG::Common::UserAgent;
 use JSON qw(from_json to_json);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.18.0';
 
 #parameter baseUrl, user, password, realm, lwpOpts
 
@@ -32,12 +32,15 @@ sub prereq {
 sub ua {
     my ($self) = @_;
     return $self->{ua} if ( $self->{ua} );
-    my $ua = Lemonldap::NG::Common::UserAgent->new();
+    my $ua = Lemonldap::NG::Common::UserAgent->new(
+        { lwpOpts => $self->{lwpOpts}, lwpSslOpts => $self->{lwpSslOpts} } );
     if ( $self->{user} ) {
         my $url  = $self->{baseUrl};
         my $port = ( $url =~ /^https/ ? 443 : 80 );
         $url =~ s#https?://([^/]*).*$#$1#;
-        $port = $1 if ( $url =~ s/:(\d+)$// );
+        if ( $url =~ s/:(\d+)$// ) {
+            $port = $1;
+        }
         $ua->credentials( "$url:$port", $self->{realm},
             $self->{user}, $self->{password} );
     }
@@ -107,6 +110,7 @@ sub store {
     $req->content( to_json($conf) );
     $req->header( 'Content-Type' => 'application/json' );
     my $resp = $self->ua->request($req);
+
     if ( $resp->is_success ) {
         my $res;
         eval { $res = from_json( $resp->content, { allow_nonref => 1 } ) };
