@@ -1,11 +1,11 @@
 package Lemonldap::NG::Handler::Lib::PSGI;
 
-#use 5.10.0;
+use strict;
 use Mouse;
 
 #use Lemonldap::NG::Handler::Main qw(:jailSharedVars);
 
-our $VERSION = '2.0.6';
+our $VERSION = '2.22.0';
 
 has protection => ( is => 'rw', isa => 'Str' );
 has rule       => ( is => 'rw', isa => 'Str' );
@@ -16,21 +16,22 @@ has api        => ( is => 'rw', isa => 'Str' );
 sub init {
     my ( $self, $args ) = @_;
     eval { $self->api->init($args) };
-    if ( $@ and not( $self->{protection} and $self->{protection} eq 'none' ) ) {
+    if ( $@ and not( $args->{protection} and $args->{protection} eq 'none' ) ) {
         $self->error($@);
         return 0;
     }
     unless ( $self->api->checkConf($self)
-        or ( $self->{protection} and $self->{protection} eq 'none' ) )
+        or ( $args->{protection} and $args->{protection} eq 'none' ) )
     {
         $self->error(
             "Unable to protect this server ($Lemonldap::NG::Common::Conf::msg)"
         );
         return 0;
     }
+
     eval { $self->portal( $self->api->tsv->{portal}->() ) };
     my $rule =
-      $self->{protection} || $self->api->localConfig->{protection} || '';
+      $args->{protection} || $self->api->localConfig->{protection} || '';
     $self->rule(
         $rule eq 'authenticate' ? 1 : $rule eq 'manager' ? '' : $rule );
     return 1;
@@ -49,10 +50,11 @@ sub _run {
 
         # Handle requests
         # Developers, be careful: Only this part is executed at each request
-        return sub {
-            return $self->_authAndTrace(
-                Lemonldap::NG::Common::PSGI::Request->new( $_[0] ) );
-        };
+        return $self->psgiAdapter(
+            sub {
+                return $self->_authAndTrace( $_[0] );
+            }
+        );
     }
 
     else {
@@ -66,33 +68,17 @@ sub _run {
         }
 
         # Handle unprotected requests
-        return sub {
-            my $req = Lemonldap::NG::Common::PSGI::Request->new( $_[0] );
-            my $res = $self->handler($req);
-            push @{ $res->[1] }, $req->spliceHdrs;
-            return $res;
-        };
+        return $self->psgiAdapter(
+            sub {
+                my $req = $_[0];
+                my $res = $self->handler($req);
+                push @{ $res->[1] }, $req->spliceHdrs;
+                return $res;
+            }
+        );
     }
 }
 
-sub status {
-    my ( $class, $args ) = @_;
-    $args //= {};
-    my $self = $class->new($args);
-    $self->init($args);
-
-    # Check if main handler initialization has been done
-    unless ( %{ $self->api->tsv } ) {
-        eval { $self->api->checkConf() };
-        $self->logger->error($@) if ($@);
-    }
-    return sub {
-        my $req = Lemonldap::NG::Common::PSGI::Request->new( $_[0] );
-        $self->api->status($req);
-        return [ 200, [ $req->spliceHdrs ], [ $req->{respBody} ] ];
-    };
-}
-
 sub reload {
     my ( $class, $args ) = @_;
     $args //= {};
@@ -104,11 +90,13 @@ sub reload {
         eval { $self->api->checkConf() };
         $self->logger->error($@) if ($@);
     }
-    return sub {
-        my $req = Lemonldap::NG::Common::PSGI::Request->new( $_[0] );
-        $self->api->reload($req);
-        return [ 200, [ $req->spliceHdrs ], [ $req->{respBody} ] ];
-    };
+    return $self->psgiAdapter(
+        sub {
+            my $req = $_[0];
+            $self->api->reload($req);
+            return [ 200, [ $req->spliceHdrs ], [ $req->{respBody} ] ];
+        }
+    );
 }
 
 ## @method private PSGI-Response _authAndTrace($req)
@@ -116,6 +104,9 @@ sub reload {
 # response is 200.
 sub _authAndTrace {
     my ( $self, $req, $noCall ) = @_;
+    unless ( eval { $self->api->tsv->{portal} } ) {
+        $self->api->checkConf(1);
+    }
 
     # TODO: handle types
     my $type = $self->api->checkType($req);
@@ -125,12 +116,11 @@ sub _authAndTrace {
     my $tmp = $self->api;
     $tmp =~ s/::\w+$/::/;
     $type = $tmp . $type;
-    eval "require $type";
+    Lemonldap::NG::Handler::Main->buildAndLoadType($type);
     die $@ if ($@);
     my ( $res, $session ) = $type->run( $req, $self->{rule} );
-    eval { $self->portal( $type->tsv->{portal}->() ) };
+    my $portal = eval { $type->tsv->{portal}->($req) };
     $self->logger->warn($@)  if $@;
-    $req->userData($session) if ($session);
 
     if ( $res < 300 ) {
         if ($noCall) {
@@ -147,7 +137,7 @@ sub _authAndTrace {
         return [ $res, [ $req->spliceHdrs ], [] ];
     }
     else {
-        my $s = ( $self->portal ? $self->portal . "/lmerror/$res" : '' );
+        my $s = ( $portal ? $portal . "/lmerror/$res" : '' );
         $s =
             '<html><head><title>Redirection</title></head><body>'
           . qq{<script type="text/javascript">window.location='$s'</script>}
@@ -186,9 +176,14 @@ sub custom {
 # @return user identifier to log
 sub userId {
     my ( $self, $req ) = @_;
-    return $req->userData->{ $Lemonldap::NG::Handler::Main::tsv->{whatToTrace}
+    my $userId =
+      $req->userData->{ $Lemonldap::NG::Handler::Main::tsv->{whatToTrace}
           || '_whatToTrace' }
+      || $req->userData->{'_user'}    # Fix 2377
       || 'anonymous';
+
+    $self->logger->debug("Returned userId: $userId");
+    return $userId;
 }
 
 ## @method boolean group(string group)
@@ -208,7 +203,7 @@ sub group {
 sub sendError {
     my ( $self, $req, $err, $code ) = @_;
     $err ||= $req->error;
-    $self->userLogger->warn( '[' . $self->userId($req) . "] $err" );
+    $self->logger->warn( '[' . $self->userId($req) . "] $err" );
     return $self->Lemonldap::NG::Common::PSGI::sendError( $req, $err, $code );
 }
 
