@@ -8,7 +8,7 @@ use Lemonldap::NG::Handler::Main;
 use Lemonldap::NG::Common::Util qw(getSameSite);
 use URI;
 
-our $VERSION = '2.18.0';
+our $VERSION = '2.22.0';
 
 ## @method hashref tests(hashref conf)
 # Return a hash ref where keys are the names of the tests and values
@@ -33,6 +33,12 @@ sub tests {
 
         # Check if portal is in domain
         portalIsInDomain => sub {
+
+            # Looks like a sub, user knows what they are doing
+            if ( $conf->{portal} =~ /[\$\(&\|"']/ ) {
+                return 1;
+            }
+
             return (
                 1,
                 (
@@ -47,16 +53,20 @@ sub tests {
         portalURL => sub {
             my $url = $conf->{portal};
 
-            # Append or remove trailing slashes
-            $conf->{portal} =~ s%/*$%/%;
-            return (
-                1,
-                (
-                    ( $url =~ m%/$% )
-                    ? ''
-                    : "Portal URL should end with a /"
-                )
-            );
+            # Looks like a sub, user knows what they are doing
+            if ( $url =~ /[\$\(&\|"']/ ) {
+                return 1;
+            }
+            else {
+                return (
+                    1,
+                    (
+                        ( $url =~ m%/$% )
+                        ? ''
+                        : "Portal URL should end with a /"
+                    )
+                );
+            }
         },
 
         # Check if virtual hosts are in the domain
@@ -84,13 +94,14 @@ sub tests {
             foreach my $vh ( keys %{ $conf->{locationRules} } ) {
                 push @pb, $vh if ( $vh =~ /:/ );
             }
-            if (@pb) {
-                return ( 0,
-                        'Virtual hosts '
-                      . join( ', ', @pb )
-                      . " contain a port, this is not allowed" );
-            }
-            else { return 1; }
+            return @pb
+              ? (
+                0,
+                'Virtual hosts '
+                  . join( ', ', @pb )
+                  . ' contain a port, this is not allowed'
+              )
+              : 1;
         },
 
         # Force vhost to be lowercase
@@ -99,13 +110,31 @@ sub tests {
             foreach my $vh ( keys %{ $conf->{locationRules} } ) {
                 push @pb, $vh if ( $vh ne lc $vh );
             }
-            if (@pb) {
-                return ( 0,
-                        'Virtual hosts '
-                      . join( ', ', @pb )
-                      . " must be in lower case" );
+            return @pb
+              ? (
+                0,
+                'Virtual hosts ' . join( ', ', @pb ) . ' must be in lower case'
+              )
+              : 1;
+        },
+
+        vhostNotAnAlias => sub {
+            my ( @pb, @aliases, @all_aliases );
+
+            @aliases =
+              map { $conf->{vhostOptions}->{$_}->{vhostAliases} }
+              keys %{ $conf->{vhostOptions} };
+
+            foreach my $alias (@aliases) {
+                push @all_aliases, split( /\s+/, $alias );
             }
-            else { return 1; }
+
+            foreach my $vh ( keys %{ $conf->{locationRules} } ) {
+                push @pb, $vh if ( grep { lc($_) eq lc($vh) } @all_aliases );
+            }
+            return @pb
+              ? ( 0, 'Virtual hosts ' . join( ', ', @pb ) . ' match an alias' )
+              : 1;
         },
 
         # Check if "userDB" and "authentication" are consistent
@@ -401,11 +430,18 @@ sub tests {
         # Test if SAML private and public keys signature keys are set
         samlSecretKeys => sub {
             return 1 unless ( $conf->{issuerDBSAMLActivation} );
+
+            if (   $conf->{samlServicePrivateKeySig}
+                && $conf->{samlServicePublicKeySig} )
+            {
+                return 1;
+            }
+            elsif ( $conf->{keys}->{"default-saml-sig"} ) {
+                return 1;
+            }
+
             return ( 0,
-                'SAML service private and public keys signature must be set' )
-              unless ( $conf->{samlServicePrivateKeySig}
-                && $conf->{samlServicePublicKeySig} );
-            return 1;
+                'SAML service private and public keys signature must be set' );
         },
 
         samlSignatureOverrideNeedsCertificate => sub {
@@ -481,6 +517,20 @@ sub tests {
               );
         },
 
+        samlIssuerNotEnabled => sub {
+            if ( keys %{ $conf->{samlSPMetaDataXML} || {} } ) {
+                if ( $conf->{issuerDBSAMLActivation} ) {
+                    return 1;
+                }
+                else {
+                    return ( 1,
+"SAML service providers require enabling the SAML Issuer in General Parameters"
+                    );
+                }
+            }
+            return 1;
+        },
+
         # Try to parse combination with declared modules
         checkCombinations => sub {
             return 1 unless ( $conf->{authentication} eq 'Combination' );
@@ -519,32 +569,20 @@ sub tests {
         # Warn if 2F dependencies seem missing
         sfaDependencies => sub {
             my $ok = 0;
-            foreach (qw(u totp utotp yubikey)) {
+            foreach (qw(totp yubikey)) {
                 $ok ||= $conf->{ $_ . '2fActivation' };
                 last if ($ok);
             }
             return 1 unless ($ok);
 
             # Use TOTP
-            if (   $conf->{totp2fActivation}
-                or $conf->{utotp2fActivation} )
-            {
+            if ( $conf->{totp2fActivation} ) {
                 eval "use Convert::Base32";
                 return ( 1,
                     "Convert::Base32 module is required to enable TOTP" )
                   if ($@);
             }
 
-            # Use U2F
-            if (   $conf->{u2fActivation}
-                or $conf->{utotp2fActivation} )
-            {
-                eval "use Crypt::U2F::Server::Simple";
-                return ( 1,
-"Crypt::U2F::Server::Simple module is required to enable U2F"
-                ) if ($@);
-            }
-
             # Use WebAuthn
             if ( $conf->{webauthn2fActivation} ) {
                 eval "use Authen::WebAuthn";
@@ -571,17 +609,6 @@ sub tests {
             return 1;
         },
 
-        # Warn if TOTP or U2F is enabled with UTOTP (U2F + TOTP)
-        utotp => sub {
-            return 1 unless ( $conf->{utotp2fActivation} );
-            my $w = "";
-            foreach ( 'totp', 'u' ) {
-                $w .= uc($_) . "2F is activated twice \n"
-                  if ( $conf->{ $_ . '2fActivation' } eq '1' );
-            }
-            return ( 1, ( $w ? $w : () ) );
-        },
-
         # Warn if TOTP not 6 or 8 digits long
         totp2fDigits => sub {
             return 1 unless ( $conf->{totp2fActivation} );
@@ -646,12 +673,6 @@ sub tests {
                   && $conf->{ $_ . '2fSelfRegistration' };
                 last if ($ok);
             }
-
-            $ok ||= $conf->{'utotp2fActivation'}
-              && ( $conf->{'u2fSelfRegistration'}
-                || $conf->{'totp2fSelfRegistration'} );
-            $msg = "A self registrable module should be enabled to require 2FA"
-              unless ($ok);
             return ( 1, $msg );
         },
 
@@ -711,10 +732,7 @@ sub tests {
         bruteForceProtection => sub {
             my @lockTimes =
               sort { $a <=> $b }
-              map {
-                $_ =~ s/\D//;
-                abs $_;
-              }
+              map  { abs(s/\D//r) }
               grep { /\d+/ }
               split /\s*,\s*/, $conf->{bruteForceProtectionLockTimes} || '';
             $conf->{bruteForceProtectionLockTimes} = join ', ', @lockTimes
@@ -769,9 +787,7 @@ sub tests {
             return 1 unless ( $conf->{disablePersistentStorage} );
             return ( 1, "2FA enabled WITHOUT persistent session storage" )
               if ( $conf->{totp2fActivation}
-                || $conf->{yubikey2fActivation}
-                || $conf->{u2fActivation}
-                || $conf->{utotp2fActivation} );
+                || $conf->{yubikey2fActivation} );
             return ( 1,
                 "History plugin enabled WITHOUT persistent session storage" )
               if ( $conf->{loginHistoryEnabled} );
@@ -826,7 +842,7 @@ sub tests {
                     ->{oidcRPMetaDataOptionsRedirectUris} )
                 {
                     push @msg,
-                      "$oidcRpId OpenID Connect RP has no redirect URI defined";
+"$oidcRpId: OIDC Relying Party has no redirect URI defined";
                     $res = 0;
                     next;
                 }
@@ -834,14 +850,42 @@ sub tests {
             return ( $res, join( ', ', @msg ) );
         },
 
-        # Test if SAML private and public keys signature keys are set
+        # OIDC public RP should implement PKCE
+        oidcRPpublicClientWithPKCE => sub {
+            return 1
+              unless ( $conf->{oidcRPMetaDataOptions}
+                and %{ $conf->{oidcRPMetaDataOptions} } );
+            my @msg;
+            foreach my $oidcRpId ( keys %{ $conf->{oidcRPMetaDataOptions} } ) {
+                if ( $conf->{oidcRPMetaDataOptions}->{$oidcRpId}
+                    ->{oidcRPMetaDataOptionsPublic} )
+                {
+                    push @msg,
+"$oidcRpId: requiring PKCE is recommended for public OIDC Relying Parties"
+                      unless $conf->{oidcRPMetaDataOptions}->{$oidcRpId}
+                      ->{oidcRPMetaDataOptionsRequirePKCE};
+                    next;
+                }
+            }
+            return ( 1, join( ', ', @msg ) );
+        },
+
+        # Test if OIDC private and public keys signature keys are set
         oidcSecretKeys => sub {
             return 1 unless ( $conf->{issuerDBOpenIDConnectActivation} );
+
+            if (   $conf->{oidcServicePrivateKeySig}
+                && $conf->{oidcServicePublicKeySig} )
+            {
+                return 1;
+            }
+            elsif ( $conf->{keys}->{"default-oidc-sig"} ) {
+                return 1;
+            }
+
             return ( 1,
-                'OIDC service private and public keys signature should be set' )
-              unless ( $conf->{oidcServicePrivateKeySig}
-                && $conf->{oidcServicePublicKeySig} );
-            return 1;
+                'OIDC service private and public keys signature should be set'
+            );
         },
 
         # RS* OIDC algs require a signing key
@@ -864,7 +908,12 @@ sub tests {
                   ->{oidcRPMetaDataOptionsAccessTokenJWT}
               } keys %{ $conf->{oidcRPMetaDataOptions} };
 
-            if ( @usingRSA and not $conf->{oidcServicePrivateKeySig} ) {
+            if (
+                @usingRSA
+                and not( $conf->{oidcServicePrivateKeySig}
+                    or $conf->{keys}->{"default-oidc-sig"} )
+              )
+            {
                 my $msg =
                   join( ", ", @usingRSA )
                   . ": using RS-type encryption, but no RSA key is defined in global OIDC configuration";
@@ -916,7 +965,7 @@ sub tests {
                   ->{oidcRPMetaDataOptionsClientID};
                 unless ($clientId) {
                     push @msg,
-                      "$clientConfKey OIDC Relying Party has no Client ID";
+                      "$clientConfKey: OIDC Relying Party has no Client ID";
                     $res = 0;
                     next;
                 }
@@ -932,6 +981,20 @@ sub tests {
             return ( $res, join( ', ', @msg ) );
         },
 
+        oidcIssuerNotEnabled => sub {
+            if ( keys %{ $conf->{oidcRPMetaDataOptions} || {} } ) {
+                if ( $conf->{issuerDBOpenIDConnectActivation} ) {
+                    return 1;
+                }
+                else {
+                    return ( 1,
+"OIDC relying parties require enabling the OpenID Connect Issuer in General Parameters"
+                    );
+                }
+            }
+            return 1;
+        },
+
         # CAS APP URL must be defined and unique
         casAppHostnameUniqueness => sub {
             return 1
@@ -954,7 +1017,7 @@ sub tests {
                     my ($appHost) = $appUrl =~ m#^(https?://[^/]+)(/.*)?$#;
                     unless ($appHost) {
                         push @msg,
-                          "$casConfKey CAS Application has no Service URL";
+                          "$casConfKey: CAS Application has no Service URL";
                         $res = 0;
                         next;
                     }
@@ -971,6 +1034,20 @@ sub tests {
             return ( $res, join( ', ', @msg ) );
         },
 
+        casIssuerNotEnabled => sub {
+            if ( keys %{ $conf->{casAppMetaDataOptions} || {} } ) {
+                if ( $conf->{issuerDBCASActivation} ) {
+                    return 1;
+                }
+                else {
+                    return ( 1,
+"CAS applications require enabling the CAS Issuer in General Parameters"
+                    );
+                }
+            }
+            return 1;
+        },
+
         # Notification system required with removed SF notification
         sfRemovedNotification => sub {
             return 1 unless ( $conf->{sfRemovedMsgRule} );
@@ -1054,7 +1131,7 @@ sub tests {
             return ( 1,
                 'FindUser wildcard should be allowed by parameters control' )
               unless (
-                $conf->{findUserWildcard} =~ /$conf->{findUserControl}/o );
+                $conf->{findUserWildcard} =~ /$conf->{findUserControl}/ );
             return 1;
         },
 
@@ -1076,6 +1153,18 @@ sub tests {
             return 1;
         },
 
+        # When auth is choice, userDB should be choice also
+        userdbChoice => sub {
+            return 1
+              unless ( $conf->{authChoiceModules}
+                and %{ $conf->{authChoiceModules} }
+                and $conf->{authentication} eq 'Choice' );
+            return ( 1, 'UserDB should be Same when authentication is Choice' )
+              unless $conf->{userDB} eq 'Same'
+              or $conf->{userDB} eq 'Choice';
+            return 1;
+        },
+
         # FindUser authChoice parameter must be defined
         findUserChoiceParam => sub {
             return ( -1, 'FindUser choice parameter must be defined' )
@@ -1145,30 +1234,6 @@ sub tests {
             return 1;
         },
 
-        # OIDC Signature and Encryption tests
-        oidcSigAlgShouldMatchKeyType => sub {
-            for my $key (
-                qw(oidcRPMetaDataOptionsIDTokenSignAlg oidcRPMetaDataOptionsAccessTokenSignAlg oidcRPMetaDataOptionsUserInfoSignAlg)
-              )
-            {
-                foreach my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} } ) {
-                    return ( 0,
-"Signature algorithm shouldn't be ES* if key type is RSA ($rp/$key)"
-                      )
-                      if $conf->{oidcRPMetaDataOptions}->{$rp}->{$key}
-                      and $conf->{oidcRPMetaDataOptions}->{$rp}->{$key} =~ /^E/
-                      and $conf->{oidcServiceKeyTypeSig} ne 'EC';
-                    return ( 0,
-"Signature algorithm shouldn't be RS* or PS* if key type is EC ($rp/$key)"
-                      )
-                      if $conf->{oidcRPMetaDataOptions}->{$rp}->{$key}
-                      and $conf->{oidcRPMetaDataOptions}->{$rp}->{$key} !~ /^E/
-                      and $conf->{oidcServiceKeyTypeSig} eq 'EC';
-                }
-            }
-            return 1;
-        },
-
 # Warn if both oidcRPMetaDataOptionsJwks and oidcRPMetaDataOptionsJwksUri is set
         noJwksDuplication => sub {
             return 1
@@ -1187,6 +1252,129 @@ sub tests {
                 "JWKS URI defined while JWKS document is fixed: "
                   . join( ', ', @pb ) );
         },
+
+# If oidcRPMetaDataOptionsAuthRequiredForAuthorize is set, oidcRPMetaDataOptionsAuthMethod should be compatible
+        oidcCompatAuth => sub {
+            return 1
+              unless $conf->{oidcRPMetaDataOptions}
+              and ref $conf->{oidcRPMetaDataOptions};
+            my @pb;
+            for my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} } ) {
+                push @pb, $rp
+                  if $conf->{oidcRPMetaDataOptions}->{$rp}
+                  ->{oidcRPMetaDataOptionsAuthRequiredForAuthorize}
+                  and $conf->{oidcRPMetaDataOptions}->{$rp}
+                  ->{oidcRPMetaDataOptionsAuthMethod}
+                  and $conf->{oidcRPMetaDataOptions}->{$rp}
+                  ->{oidcRPMetaDataOptionsAuthMethod} !~
+                  /^(?:client_secret|private_key)_jwt$/;
+            }
+            return 1 unless @pb;
+            return ( 1,
+                    'Incompatible required authentication methods in RP '
+                  . '(only client_secret_jwt and private_key_jwt are allowed '
+                  . 'when authentication is required on authorization endpoint: '
+                  . join( ', ', @pb ) );
+        },
+
+        # Test password length policy if enabled
+        ppMaxSizeGreaterThanMinSize => sub {
+            return ( 1,
+                'Password maximum size should be greater than minimal size' )
+              if ( (
+                    $conf->{passwordPolicyMinSize} >=
+                    $conf->{passwordPolicyMaxSize}
+                )
+                and $conf->{passwordPolicyMaxSize}
+                and $conf->{passwordPolicyActivation}
+              );
+            return 1;
+        },
+
+        # Test password minimal size policy if enabled
+        ppMinSize => sub {
+            my $total;
+            foreach (qw(Lower Upper Digit SpeChar)) {
+                $total += $conf->{"passwordPolicyMin$_"}
+                  if $conf->{"passwordPolicyMin$_"} > 0;
+            }
+            return ( 1,
+'Password minimal size should be greater than total of minimal sizes'
+              )
+              if (  ( $conf->{passwordPolicyMinSize} < $total )
+                and $conf->{passwordPolicyMinSize}
+                and $conf->{passwordPolicyActivation} );
+            return 1;
+        },
+        accessTokenConsistency => sub {
+            return 1
+              unless $conf->{issuerDBOpenIDConnectActivation};
+            my @pb;
+            foreach my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} || {} } ) {
+                my $opts = $conf->{oidcRPMetaDataOptions}->{$rp};
+                push @pb, $rp
+                  if $opts->{oidcRPMetaDataOptionsAccessTokenClaims}
+                  and not $opts->{oidcRPMetaDataOptionsAccessTokenJWT};
+            }
+            return 1 unless @pb;
+            return ( 0,
+                'access_token cannot be opaque with claims in access_token ('
+                  . join( ', ', @pb )
+                  . ')' );
+        },
+        oidcNoneConsistency => sub {
+            my @pb;
+            return 1 unless $conf->{oidcServiceMetaDataDisallowNoneAlg};
+            foreach my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} || {} } ) {
+                my $opts = $conf->{oidcRPMetaDataOptions}->{$rp};
+                push @pb, $rp
+                  if $opts->{oidcRPMetaDataOptionsIDTokenSignAlg} eq 'none'
+                  or $opts->{oidcRPMetaDataOptionsUserInfoSignAlg} eq 'none';
+            }
+            return 1 unless @pb;
+            return ( 1,
+                'Signature algorithm is not allowed but set for: '
+                  . join( ', ', @pb ) );
+        },
+        oidcNativeSso => sub {
+            return ( 0, 'Native SSO without OIDC identity service' )
+              if $conf->{oidcServiceAllowNativeSso}
+              and not $conf->{issuerDBOpenIDConnectActivation};
+            return 1
+              unless $conf->{oidcRPMetaDataOptions}
+              and ref $conf->{oidcRPMetaDataOptions};
+            my @needNativeSso;
+            if ( $conf->{oidcRPMetaDataOptions}
+                and ref $conf->{oidcRPMetaDataOptions} )
+            {
+                for my $rp ( keys %{ $conf->{oidcRPMetaDataOptions} } ) {
+                    push @needNativeSso, $rp
+                      if $conf->{oidcRPMetaDataOptions}->{$rp}
+                      ->{oidcRPMetaDataOptionsAllowNativeSso};
+                }
+            }
+            if ( @needNativeSso and not $conf->{oidcServiceAllowNativeSso} ) {
+                return ( 1,
+                    "Native SSO isn't enabled but needed by: "
+                      . join( ', ', @needNativeSso ) );
+            }
+            if ( !@needNativeSso and $conf->{oidcServiceAllowNativeSso} ) {
+                return ( 1, 'Native SSO service enabled but useless' );
+            }
+            return 1;
+        },
+        crowdSecBouncer => sub {
+            return 1 unless $conf->{crowdsec};
+            return 1 if $conf->{crowdsecKey};
+            return ( 0, 'Crowdsec Bouncer enabled without API key' );
+        },
+        crowdSecAgent => sub {
+            return 1 unless $conf->{crowdsecAgent};
+            return 1
+              if $conf->{crowdsecMachineId} and $conf->{crowdsecPassword};
+            return ( 0,
+                'Crowdsec Agent enabled without machine_id and password' );
+        },
     };
 }
 
