@@ -14,7 +14,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_IMPERSONATION_SERVICE_NOT_ALLOWED
 );
 
-our $VERSION = '2.0.10';
+our $VERSION = '2.22.0';
 
 extends qw(
   Lemonldap::NG::Portal::Main::Plugin
@@ -38,9 +38,19 @@ has rule                  => ( is => 'rw', default => sub { 0 } );
 has idRule                => ( is => 'rw', default => sub { 1 } );
 has unrestrictedUsersRule => ( is => 'rw', default => sub { 0 } );
 
+# Prefixed sessions id attribute
+has prefixedSId => (
+    is      => 'ro',
+    lazy    => 1,
+    default => sub {
+        my $self = $_[0];
+        return $self->conf->{contextSwitchingPrefix} . '_session_id';
+    }
+);
+
 sub init {
     my ($self) = @_;
-    $self->addAuthRoute( switchcontext => 'run',  ['POST'] )
+    $self->addAuthRoute( switchcontext => 'run', ['POST'] )
       ->addAuthRoute( switchcontext => 'display', ['GET'] );
 
     # Parse ContextSwitching rules
@@ -76,29 +86,23 @@ sub init {
 sub display {
     my ( $self, $req ) = @_;
     my ( $realSession, $realSessionId );
-    if ( $realSessionId =
-        $req->userData->{"$self->{conf}->{contextSwitchingPrefix}_session_id"} )
-    {
+    if ( $realSessionId = $req->userData->{ $self->prefixedSId } ) {
         unless ( $realSession = $self->p->getApacheSession($realSessionId) ) {
             $self->userLogger->info(
                 "ContextSwitching: session $realSessionId expired");
-            return $self->p->do( $req, [ sub { PE_SESSIONEXPIRED } ] );
+            return $self->p->doPE( $req, PE_SESSIONEXPIRED );
         }
     }
 
     # Check access rules
     unless ( $self->rule->( $req, $req->userData )
-        || $req->userData->{
-            "$self->{conf}->{contextSwitchingPrefix}_session_id"} )
+        || $req->userData->{ $self->prefixedSId } )
     {
         $self->userLogger->warn('ContextSwitching service NOT authorized');
-        return $self->p->do( $req,
-            [ sub { PE_IMPERSONATION_SERVICE_NOT_ALLOWED } ] );
+        return $self->p->doPE( $req, PE_IMPERSONATION_SERVICE_NOT_ALLOWED );
     }
 
-    if (
-        $req->userData->{"$self->{conf}->{contextSwitchingPrefix}_session_id"} )
-    {
+    if ( $req->userData->{ $self->prefixedSId } ) {
         $self->logger->debug('Request to stop ContextSwitching');
         if ( $self->conf->{contextSwitchingStopWithLogout} ) {
             $self->userLogger->notice("Stop ContextSwitching for $req->{user}");
@@ -113,16 +117,13 @@ sub display {
             $req = $self->_abortImpersonation( $req, $req->{user},
                 $realSession->data->{ $self->conf->{whatToTrace} }, 0 );
             $self->p->updateSession( $req, $req->userData );
-            return $self->p->do( $req, [ sub { PE_REDIRECT } ] );
+            return $self->p->doPE( $req, PE_REDIRECT );
         }
     }
 
     # Display form
     my $params = {
-        PORTAL        => $self->conf->{portal},
-        MAIN_LOGO     => $self->conf->{portalMainLogo},
-        SKIN          => $self->p->getSkin($req),
-        LANGS         => $self->conf->{showLanguages},
+        FORM_ACTION   => $self->p->relativeUrl( $req, 'switchcontext' ),
         MSG           => 'contextSwitching_ON',
         ALERTE        => 'alert-danger',
         IMPERSONATION => $self->conf->{contextSwitchingRule},
@@ -138,22 +139,22 @@ sub display {
 
 sub run {
     my ( $self, $req ) = @_;
-    my $statut = PE_OK;
-    my $realId = $req->userData->{ $self->conf->{whatToTrace} };
+    my $statut  = PE_OK;
+    my $realId  = $req->userData->{ $self->conf->{whatToTrace} };
     my $spoofId = $req->param('spoofId') || '';    # ContextSwitching required ?
-    my $unUser = $self->unrestrictedUsersRule->( $req, $req->userData ) || 0;
+    my $unUser  = $self->unrestrictedUsersRule->( $req, $req->userData ) || 0;
 
     # Check token
     if ( $self->ottRule->( $req, {} ) ) {
         my $token = $req->param('token');
         unless ($token) {
             $self->userLogger->warn('ContextSwitching called without token');
-            return $self->p->do( $req, [ sub { PE_NOTOKEN } ] );
+            return $self->p->doPE( $req, PE_NOTOKEN );
         }
         unless ( $self->ott->getToken($token) ) {
             $self->userLogger->warn(
                 'ContextSwitching called with an expired/bad token');
-            return $self->p->do( $req, [ sub { PE_TOKENEXPIRED } ] );
+            return $self->p->doPE( $req, PE_TOKENEXPIRED );
         }
     }
 
@@ -161,24 +162,23 @@ sub run {
     unless ( $self->rule->( $req, $req->userData ) ) {
         $self->userLogger->warn('ContextSwitching service NOT authorized');
         $spoofId = '';
-        return $self->p->do( $req,
-            [ sub { PE_IMPERSONATION_SERVICE_NOT_ALLOWED } ] );
+        return $self->p->doPE( $req, PE_IMPERSONATION_SERVICE_NOT_ALLOWED );
     }
 
     # ContextSwitching required -> Check user Id
     if ( $spoofId && $spoofId ne $req->{user} ) {
         $self->logger->debug("Spoofed Id: $spoofId");
-        unless ( $spoofId =~ /$self->{conf}->{userControl}/o ) {
+        unless ( $spoofId =~ /$self->{conf}->{userControl}/ ) {
             $self->userLogger->warn('Malformed spoofed Id');
             $self->logger->debug(
                 "ContextSwitching tried with spoofed Id: $spoofId");
-            return $self->p->do( $req, [ sub { PE_MALFORMEDUSER } ] );
+            return $self->p->doPE( $req, PE_MALFORMEDUSER );
         }
     }
     else {
         $self->logger->debug("contextSwitching NOT required");
-        $req->urldc( $self->conf->{portal} );
-        return $self->p->do( $req, [ sub { PE_OK } ] );
+        $req->urldc( $req->portal );
+        return $self->p->doPE( $req, PE_OK );
     }
 
     # Create spoofed session
@@ -194,7 +194,7 @@ sub run {
     );
 
     $req->mustRedirect(1);
-    return $self->p->do( $req, [ sub { $statut } ] );
+    return $self->p->doPE( $req, $statut );
 }
 
 sub _switchContext {
@@ -236,8 +236,7 @@ sub _switchContext {
         $raz = 1;
     }
 
-    $req->sessionInfo->{"$self->{conf}->{contextSwitchingPrefix}_session_id"} =
-      $realSessionId;
+    $req->sessionInfo->{ $self->prefixedSId } = $realSessionId;
 
     return $self->_abortImpersonation( $req, $spoofId, $realId, 1 ) if $raz;
 
@@ -262,9 +261,8 @@ sub _switchContext {
 
 sub _abortImpersonation {
     my ( $self, $req, $spoofId, $realId, $abort ) = @_;
-    my $type = $abort ? 'sessionInfo' : 'userData';
-    my $realSessionId =
-      $req->{$type}->{"$self->{conf}->{contextSwitchingPrefix}_session_id"};
+    my $type          = $abort ? 'sessionInfo' : 'userData';
+    my $realSessionId = $req->{$type}->{ $self->prefixedSId };
     my $session;
     unless ( $session = $self->p->getApacheSession($realSessionId) ) {
         $self->userLogger->info("Session $session expired");
@@ -291,11 +289,10 @@ sub _abortImpersonation {
     # Restore real session
     $req->{$type} = { %{ $session->data } };
     $req->{user} = $session->data->{_user};
-    $req->urldc( $self->conf->{portal} );
+    $req->urldc( $req->portal );
     $req->id($realSessionId);
     $self->p->buildCookie($req);
-    delete $req->{$type}
-      ->{"$self->{conf}->{contextSwitchingPrefix}_session_id"};
+    delete $req->{$type}->{ $self->prefixedSId };
 
     return $req;
 }
@@ -303,7 +300,7 @@ sub _abortImpersonation {
 sub displayLink {
     my ( $self, $req ) = @_;
     return 'OFF'
-      if $req->userData->{"$self->{conf}->{contextSwitchingPrefix}_session_id"};
+      if $req->userData->{ $self->prefixedSId };
     return 'ON' if $self->rule->( $req, $req->userData );
 }
 
