@@ -4,32 +4,32 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants;
 
-extends 'Lemonldap::NG::Common::Module';
+extends 'Lemonldap::NG::Portal::Main::UserDB';
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.19.0';
 
 # INITIALIZATION
 
 sub init {
-    1;
+    return 1;
 }
 
 # RUNNING METHODS
 
 sub getUser {
-    PE_OK;
+    return PE_OK;
 }
 
 sub findUser {
-    PE_OK;
+    return PE_OK;
 }
 
 sub setSessionInfo {
-    PE_OK;
+    return PE_OK;
 }
 
 sub setGroups {
-    PE_OK;
+    return PE_OK;
 }
 
 1;
