@@ -13,25 +13,27 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_FORBIDDENIP
 );
 
-our $VERSION = '2.0.10';
+our $VERSION = '2.19.0';
 
 extends qw(
-  Lemonldap::NG::Common::Module
+  Lemonldap::NG::Portal::Main::UserDB
   Lemonldap::NG::Portal::Lib::Slave
 );
 
 # INITIALIZATION
 
-sub init { 1 }
+sub init {
+    return 1;
+}
 
 # RUNNING METHODS
 
 sub getUser {
-    PE_OK;
+    return PE_OK;
 }
 
 sub findUser {
-    PE_OK;
+    return PE_OK;
 }
 
 # Search exportedVars values in HTTP headers.
@@ -53,8 +55,4 @@ sub setSessionInfo {
     return PE_OK;
 }
 
-sub setGroups {
-    PE_OK;
-}
-
 1;
